/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Basic implementation of the {@link PrimitiveCompiler} interface. This
 * implementation delegates definition compilation requests to a set of
 * {@link PrimitiveCompiler primitive-compilers} according to the alphabetical
 * order of their client interface names.
 */
public class PrimitiveCompilerDispatcher
    implements
      PrimitiveCompiler,
      BindingController {
  /**
   * Name of the collection interface bound to the
   * {@link PrimitiveCompiler PrimitiveCompilers} used by this compiler.
   */
  public static final String            PRIMITIVE_COMPILERS_BINDING = "client-compiler";

  /**
   * The primitive compilers used by this compiler.
   */
  public Map<String, PrimitiveCompiler> primitiveCompilersItf       = new TreeMap<String, PrimitiveCompiler>();

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {
    for (final PrimitiveCompiler compiler : primitiveCompilersItf.values()) {
      compiler.compile(path, container, tasks, context);
    }
  }

  // --------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // --------------------------------------------------------------------------

  public String[] listFc() {
    return primitiveCompilersItf.keySet().toArray(
        new String[primitiveCompilersItf.size()]);
  }

  public Object lookupFc(final String itf) {
    if (itf.startsWith(PRIMITIVE_COMPILERS_BINDING)) {
      return primitiveCompilersItf.get(itf);
    }
    return null;
  }

  public void bindFc(final String itf, final Object value) {
    if (itf.startsWith(PRIMITIVE_COMPILERS_BINDING)) {
      primitiveCompilersItf.put(itf, (PrimitiveCompiler) value);
    }
  }

  public void unbindFc(final String itf) {
    if (itf.startsWith(PRIMITIVE_COMPILERS_BINDING)) {
      primitiveCompilersItf.remove(itf);
    }
  }

}
