/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.dot;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.components.Component;
import org.objectweb.fractal.adl.Compiler;
import org.objectweb.fractal.task.codegen.api.FactoryProviderTask;
import org.objectweb.fractal.task.codegen.api.FileCollectionProviderTask;
import org.objectweb.fractal.task.codegen.api.FileProviderTask;
import org.objectweb.fractal.task.codegen.api.InstanceProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireFactoryProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireFileCollectionProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireFileProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireInstanceProviderTask;
import org.objectweb.fractal.task.codegen.api.RequireSourceCodeProviderTask;
import org.objectweb.fractal.task.codegen.api.SourceCodeProviderTask;
import org.objectweb.fractal.task.codegen.lib.AbstractRecursiveFileProviderTask;
import org.objectweb.fractal.task.codegen.lib.AbstractRecursiveSourceCodeProviderTask;
import org.objectweb.fractal.task.core.BasicTaskMap;
import org.objectweb.fractal.task.core.Task;
import org.objectweb.fractal.task.core.TaskMap;
import org.objectweb.fractal.task.core.BasicTaskMap.Key;

/**
 * Generates a .dot file representing the graph of task to be executed to
 * compile the definition.
 */
public class TaskmapToDot implements Compiler {

  public void compile(final Definition definition, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {
    final BasicTaskMap taskMap = (BasicTaskMap) tasks;
    final Map<TaskMap.TaskHole, Map.Entry<BasicTaskMap.Key, TaskMap.TaskHole>> reverseMap = new HashMap<TaskMap.TaskHole, Entry<Key, TaskMap.TaskHole>>();
    for (final Map.Entry<BasicTaskMap.Key, TaskMap.TaskHole> entry : taskMap
        .entrySet()) {
      reverseMap.put(entry.getValue(), entry);
    }
    final Map<Task, String> taskName = new HashMap<Task, String>();
    int index = 0;
    for (final TaskMap.TaskHole taskHole : taskMap.values()) {
      if (taskHole.isOptional()) continue;
      taskName.put(taskHole.getTask(), "task" + (index++));
    }

    final File adlOutDir = (File) context.get("adlBuildDirectory");
    final String fileName = definition.getName().replace('.',
        File.separatorChar)
        + ".tasks.dot";
    final File dir = (fileName.lastIndexOf(File.separatorChar) == -1)
        ? adlOutDir
        : new File(adlOutDir, fileName.substring(0, fileName
            .lastIndexOf(File.separatorChar)));
    final File file = new File(adlOutDir, fileName);
    try {
      dir.mkdirs();
      final PrintStream out = new PrintStream(new FileOutputStream(file));
      out.println("// THIS FILE HAS BEEN GENERATED BY THE CECILIA-ADL COMPILER.");
      out.println("// DO NOT EDIT");
      out.println();

      out.print("digraph " + definition.getName().replace('.', '_'));
      out.println("{");
      out.println("  rankdir=LR");

      for (final Map.Entry<BasicTaskMap.Key, TaskMap.TaskHole> entry : taskMap
          .entrySet()) {
        // if task hole is optional, pass it
        if (entry.getValue().isOptional()) continue;
        final BasicTaskMap.Key key = entry.getKey();
        final Task task = entry.getValue().getTask();
        final String name = taskName.get(task);
        final StringBuffer sb = new StringBuffer(name);
        sb.append(" [shape=record, label=\"\\\"");

        // add task name
        sb.append(key.getType().replaceAll("\\.", "\\\\.")).append("\\\" | ");

        // add task id
        final Object id = key.getId();
        if (id instanceof Definition)
          sb.append(((Definition) id).getName().replaceAll("\\.", "\\\\."));
        else if (id instanceof Component)
          sb.append(((Component) id).getName().replaceAll("\\.", "\\\\."));
        else if (id instanceof File)
          sb.append(((File) id).getName().replaceAll("\\.", "\\\\."));
        else if (id instanceof FactoryProviderTask)
          sb.append(((FactoryProviderTask) id).getFactory().toString()
              .replaceAll("\\.", "\\\\."));
        else if (id != null)
          sb.append(id.toString().replaceAll("\\.", "\\\\."));
        sb.append(" | ");

        if (task instanceof SourceCodeProviderTask)
          if (task instanceof AbstractRecursiveSourceCodeProviderTask)
            sb.append("RecursiveSourceCodeProviderTask");
          else
            sb.append("SourceCodeProviderTask");
        else if (task instanceof FileProviderTask)
          if (task instanceof AbstractRecursiveFileProviderTask)
            sb.append("RecursiveFileProviderTask");
          else
            sb.append("FileProviderTask");
        else if (task instanceof FileCollectionProviderTask)
          if (task instanceof RequireFileCollectionProviderTask)
            sb.append("RecursiveFileCollectionProviderTask");
          else
            sb.append("FileCollectionProviderTask");
        else if (task instanceof FactoryProviderTask)
          sb.append("FactoryProviderTask");
        else if (task instanceof InstanceProviderTask)
          sb.append("InstanceProviderTask");
        sb.append("\"];");
        out.println(sb);
      }

      for (final Map.Entry<BasicTaskMap.Key, TaskMap.TaskHole> entry : taskMap
          .entrySet()) {
        // if task hole is optional, pass it
        if (entry.getValue().isOptional()) continue;
        final Task task = entry.getValue().getTask();
        final Set<Task> previous = new HashSet<Task>();
        for (final TaskMap.TaskHole taskHole : task.getPreviousTasks()) {
          // if task hole is optional, pass it
          if (taskHole.isOptional()) continue;
          previous.add(taskHole.getTask());
        }

        final String name = taskName.get(task);
        final StringBuffer sb = new StringBuffer();

        if (task instanceof RequireFactoryProviderTask) {
          final FactoryProviderTask t = ((RequireFactoryProviderTask) task)
              .getFactoryProviderTask();
          if (t != null) {
            sb.append(name).append(" -> ");
            sb.append(taskName.get(t)).append("[style=dashed, samehead];\n");
            previous.remove(t);
          }
        }
        if (task instanceof RequireInstanceProviderTask) {
          final InstanceProviderTask t = ((RequireInstanceProviderTask) task)
              .getInstanceProviderTask();
          if (t != null) {
            sb.append(name).append(" -> ");
            sb.append(taskName.get(t)).append("[style=dashed, samehead];\n");
            previous.remove(t);
          }
        }
        if (task instanceof RequireSourceCodeProviderTask) {
          final RequireSourceCodeProviderTask rscpt = (RequireSourceCodeProviderTask) task;
          for (final String providerName : rscpt
              .getSourceCodeProviderTaskNames()) {
            final SourceCodeProviderTask t = rscpt
                .getSourceCodeProviderTask(providerName);
            sb.append(name).append(" -> ").append(taskName.get(t)).append(
                " [color=blue, style=bold];\n");
            previous.remove(t);
          }
        }

        if (task instanceof RequireFileProviderTask) {
          final RequireFileProviderTask rfpt = (RequireFileProviderTask) task;
          for (final FileProviderTask fpt : rfpt.getFileProviderTasks()) {
            sb.append(name).append(" -> ").append(taskName.get(fpt)).append(
                " [color=blueviolet, style=bold];\n");
            previous.remove(fpt);
          }
        }

        if (task instanceof RequireFileCollectionProviderTask) {
          final RequireFileCollectionProviderTask rfcpt = (RequireFileCollectionProviderTask) task;
          for (final FileCollectionProviderTask fcpt : rfcpt
              .getFileCollectionProviderTasks()) {
            sb.append(name).append(" -> ").append(taskName.get(fcpt)).append(
                " [color=violet, style=bold];\n");
            previous.remove(fcpt);
          }
        }

        if (!previous.isEmpty()) {
          sb.append(name).append(" -> { ");
          for (final Task taskHole : previous)
            sb.append(taskName.get(taskHole)).append("; ");
          sb.append("}");
        }
        out.println(sb);
      }
      out.println("}");
      out.close();

    } catch (final IOException e) {
      throw new ADLException("An error occurs while writing to file '"
          + file.getAbsolutePath() + "'", null, e);
    }
  }
}
